package com.marcpg.bbm.lobby;

import com.google.common.io.ByteArrayDataOutput;
import com.google.common.io.ByteStreams;
import com.marcpg.bbm.lobby.item.ClickItem;
import com.marcpg.bbm.lobby.item.ItemBuilder;
import com.mojang.brigadier.arguments.StringArgumentType;
import com.mojang.brigadier.builder.LiteralArgumentBuilder;
import com.mojang.brigadier.builder.RequiredArgumentBuilder;
import com.mojang.brigadier.tree.LiteralCommandNode;
import io.papermc.paper.command.brigadier.CommandSourceStack;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.format.TextDecoration;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.event.inventory.InventoryType;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;
import xyz.xenondevs.invui.gui.Gui;
import xyz.xenondevs.invui.window.Window;

import java.util.List;

public final class Compass implements Listener {
    private final Gui gui;
    private final ItemStack item = new ItemBuilder(Material.COMPASS)
            .name(Component.text("Gamemode Compass", NamedTextColor.YELLOW).decoration(TextDecoration.BOLD, false))
            .build();

    public Compass() {
        List<Component> lore = List.of(Component.text("Click to play build battles.", NamedTextColor.BLUE, TextDecoration.UNDERLINED).decoration(TextDecoration.ITALIC, false));
        gui = Gui.normal().setStructure(
                        "# # # # L # # # #",
                        ". . . . . . . . .",
                        ". . B b S F C . .",
                        ". . . . . . . . .",
                        "# # # # # # # # #")
                .addIngredient('#', new ItemBuilder(Material.LIGHT_BLUE_STAINED_GLASS_PANE).name(Component.empty()).build())
                .addIngredient('L', new ClickItem(new ItemBuilder(Material.NETHER_STAR)
                        .name(Component.text("Lobby", NamedTextColor.WHITE, TextDecoration.BOLD).decoration(TextDecoration.ITALIC, false))
                        .lore(lore).build(), (p, e) -> send(p, "lobby")))
                .addIngredient('B', new ClickItem(new ItemBuilder(Material.RED_BED)
                        .name(Component.text("Bed Wars", NamedTextColor.DARK_RED, TextDecoration.BOLD).decoration(TextDecoration.ITALIC, false))
                        .lore(lore).build(), (p, e) -> send(p, "bedwars")))
                .addIngredient('b', new ClickItem(new ItemBuilder(Material.OAK_PLANKS)
                        .name(Component.text("Build Battles", NamedTextColor.BLUE, TextDecoration.BOLD).decoration(TextDecoration.ITALIC, false))
                        .lore(lore).build(), (p, e) -> send(p, "buildbattle")))
                .addIngredient('S', new ClickItem(new ItemBuilder(Material.GRAY_CONCRETE)
                        .name(Component.text("Soon - Skyblock", NamedTextColor.DARK_GREEN, TextDecoration.BOLD).decoration(TextDecoration.ITALIC, false))
                        .lore(lore).build(), (p, e) -> p.sendMessage(Component.text("You cannot play Skyblock yet!", NamedTextColor.RED))))
                .addIngredient('F', new ClickItem(new ItemBuilder(Material.DIAMOND_SWORD)
                        .name(Component.text("Factions", NamedTextColor.RED, TextDecoration.BOLD).decoration(TextDecoration.ITALIC, false))
                        .lore(lore).build(), (p, e) -> send(p, "factions")))
                .addIngredient('C', new ClickItem(new ItemBuilder(Material.COMMAND_BLOCK)
                        .name(Component.text("Creative", NamedTextColor.GOLD, TextDecoration.BOLD).decoration(TextDecoration.ITALIC, false))
                        .lore(lore).build(), (p, e) -> send(p, "creative")))
                .build();
    }

    @EventHandler(ignoreCancelled = true)
    public void onPlayerJoin(@NotNull PlayerJoinEvent event) {
        event.getPlayer().getInventory().setItem(4, item);
        event.getPlayer().getInventory().setHeldItemSlot(4);
    }

    @EventHandler()
    public void onPlayerInteract(@NotNull PlayerInteractEvent event) {
        if ((event.getAction() == Action.RIGHT_CLICK_AIR || event.getAction() == Action.RIGHT_CLICK_BLOCK) && event.getItem() != null && event.getItem().getType() == Material.COMPASS) {
            Window.single().setTitle("Gamemode Compass").setGui(gui).open(event.getPlayer());
        } else {
            event.setCancelled(true);
        }
    }

    @EventHandler(ignoreCancelled = true)
    public void onInventoryClick(@NotNull InventoryClickEvent event) {
        if (event.getSlotType() == InventoryType.SlotType.QUICKBAR)
            event.setCancelled(true);
    }

    public static void send(@NotNull Player player, String server) {
        ByteArrayDataOutput out = ByteStreams.newDataOutput();
        out.writeUTF("Connect");
        out.writeUTF(server);
        player.sendPluginMessage(BBMLobby.PLUGIN, "BungeeCord", out.toByteArray());
    }

    @SuppressWarnings("UnstableApiUsage")
    public static LiteralCommandNode<CommandSourceStack> joinCommand() {
        return LiteralArgumentBuilder.<CommandSourceStack>literal("lobby-join")
                .requires(source -> source.getSender() instanceof Player)
                .then(RequiredArgumentBuilder.<CommandSourceStack, String>argument("server", StringArgumentType.word())
                        .executes(context -> {
                            send((Player) context.getSource().getSender(), context.getArgument("server", String.class));
                            return 1;
                        })
                )
                .build();
    }
}
