package com.marcpg;

import com.marcpg.libpg.data.time.Time;
import com.mojang.brigadier.builder.LiteralArgumentBuilder;
import com.mojang.brigadier.tree.LiteralCommandNode;
import com.velocitypowered.api.command.CommandSource;
import com.velocitypowered.api.event.Subscribe;
import com.velocitypowered.api.event.connection.DisconnectEvent;
import com.velocitypowered.api.event.connection.PostLoginEvent;
import net.kyori.adventure.text.Component;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;

public class ServerStats {
    public static final HashMap<UUID, Long> players = new HashMap<>();
    public static final List<Long> playtimes = new ArrayList<>();

    public static int totalJoined;
    public static int maxOnlinePlayers;
    public static long maxOnlineDate;

    @Subscribe
    public void onPostLogin(@NotNull PostLoginEvent event) {
        players.put(event.getPlayer().getUniqueId(), System.currentTimeMillis());
        totalJoined++;
        updateMaxPlayers();
    }

    @Subscribe
    public void onDisconnect(@NotNull DisconnectEvent event) {
        playtimes.add(System.currentTimeMillis() - players.get(event.getPlayer().getUniqueId()));
    }


    public void updateMaxPlayers() {
        if (BBMProxy.SERVER.getAllPlayers().size() > maxOnlinePlayers) {
            maxOnlinePlayers = BBMProxy.SERVER.getAllPlayers().size();
            maxOnlineDate = System.currentTimeMillis();
        }
    }

    public static LiteralCommandNode<CommandSource> serverStatsCommand() {
        return LiteralArgumentBuilder.<CommandSource>literal("server-stats")
                .executes(context -> {
                    long avgPlaytime = playtimes.stream()
                            .mapToLong(p -> p)
                            .sum() / (long) playtimes.size();

                    context.getSource().sendMessage(Component.text("Total joined: ", Colors.RESET).append(Component.text(totalJoined, Colors.ACCENT)));
                    context.getSource().sendMessage(Component.text("Unique players: ", Colors.RESET).append(Component.text(players.size(), Colors.ACCENT)));
                    context.getSource().sendMessage(Component.text("AVG Playtime: ", Colors.RESET).append(Component.text(new Time(avgPlaytime / 1000).getPreciselyFormatted(), Colors.ACCENT)));

                    return 1;
                })
                .build();
    }
}
